var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-02CF7021-71FF-4F31-A072-291EF86CCA5A\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>FBX Nodes</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-02CF7021-71FF-4F31-A072-291EF86CCA5A\"></a><div class=\"head\">\n            <h1>FBX Nodes</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_ACFD42E4BA124E66BEA8B31451B3D933\"></a> \n            <h2><a name=\"GUID-A618B47D-FF32-494D-AB44-79ACC822B3F1\"></a>What is a Node?\n            </h2> \n            <p><a name=\"GUID-CBFB6D4C-A36E-4479-800C-A58149B5605D\"></a>Nodes are primarily used to specify the position, rotation and scale of scene elements\n               within a scene. Nodes are abstracted by the \n               		  <span class=\"code\">FbxNode</span> class. A \n               		  <span class=\"code\">FbxScene</span> contains a parent-child hierarchy of nodes. The root node of this tree is accessed\n               via \n               		  <span class=\"code\">FbxScene::GetRootNode()</span>. As detailed in \n               		  <span class=\"char_link\"><a href=\"GUID-4F644045-380D-4B75-A2A3-D39DDE53BEDD.htm\">FBX Scenes</a></span>, additional nodes can be created and added to this root node. \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>// Get the root node of the scene.\nFbxNode* lRootNode = lScene-&gt;GetRootNode();\n\n// Create a child node.\nFbxNode* lNode = FbxNode::Create(lScene, \"child\");\n\n// Add the child to the root node.\nlRootNode-&gt;AddChild(lNode);</pre></div> \n            <div><a name=\"GUID-34F1A9D2-8ED9-4EAC-8B0D-824F79829532\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>Some file formats do not allow for duplicate node names. In this case, the duplicate\n                  names will be renamed to: \n                  		  <span class=\"code\">nodename_ncl1</span>, \n                  		  <span class=\"code\">nodename_ncl2</span>, \n                  		  <span class=\"code\">nodename_ncl3</span>, etc. Note that previous FBX file formats (version 1.0 to version 6.1) did not allow\n                  for duplicate node names. \n                  		\n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_A246859352E5461D8CDBA2E94C1978BE\"></a> \n            <h2><a name=\"GUID-98A6CEB0-8A42-4F56-A29E-7DD14F6C3351\"></a>Node Hierarchy\n            </h2> \n            <p><a name=\"GUID-F29BBEF9-4777-4E15-969F-47BAA734C5E8\"></a>The node hierarchy is traversed using methods such as \n               		  <span class=\"code\">FbxNode::GetChild()</span> and \n               		  <span class=\"code\">FbxNode::GetParent()</span>. \n               		  <span class=\"code\">FbxNode::GetChildCount()</span> returns the number of children of that node. \n               		\n            </p> \n            <p><a name=\"GUID-02155C42-E845-43E9-9504-4CE7AB3FC7AC\"></a>Nodes are organized in a hierarchy such that the position, rotation and scale of a\n               node is described in relation to its parent\'s coordinate system. For example, in the\n               diagram below, if the \n               		  <span class=\"code\">cubeNode</span> is translated by 4 units along the \n               		  <span class=\"code\">rootNode</span>\'s x-axis, the \n               		  <span class=\"code\">lightNode</span> will also be affected by this translation. However, \n               		  <span class=\"code\">cameraNode</span> will not be affected by this translation because \n               		  <span class=\"code\">cameraNode</span> is not a child of \n               		  <span class=\"code\">cubeNode</span>. \n               		\n            </p> \n            <p><a name=\"GUID-91F46FAB-717F-45DC-A3F4-282FA8B589F1\"></a>The order in which the rotation and scaling transforms are applied to a parent and\n               its children is specified by the node\'s inherit type (<span class=\"code\">ETransformInheritType</span>). This transformation inheritance can be set using \n               		  <span class=\"code\">FbxNode::SetTransformationInheritType()</span>. Consult the \n               		  <span class=\"code\">FbxNode</span> class documentation for more details. \n               		\n            </p> \n            <p><a name=\"GUID-401DBF08-3263-4E02-94BB-C3F8C8D801C0\"></a></p> <img src=\"../images/GUID-11D715D7-470E-4A19-B7CC-77B8A6B5A6B6-low.png\"> \n            <p><a name=\"GUID-D0B85FC8-D253-445E-ABCA-80A4A2427EDB\"></a>As mentioned in \n               		  <span class=\"char_link\"><a href=\"GUID-4E8F95CA-CD9B-4E24-B8AD-7C399EA78137.htm#SECTION_1AF3CFAE0F43472B8D09D7471B62AE1B\">Exporting a Scene</a></span> and \n               		  <span class=\"char_link\"><a href=\"GUID-4F644045-380D-4B75-A2A3-D39DDE53BEDD.htm#SECTION_80E2BF8B6C094C09AA34E21926AF5591\">FBX Scenes</a></span>, the root node of the scene is not saved when the scene is exported. Only the recursively\n               descendant children of the root node are exported with the scene. In the diagram above,\n               assume \n               		  <span class=\"code\">rootNode</span> is the root node of a scene. When the scene is exported, \n               		  <span class=\"code\">rootNode</span> will not be saved. However, \n               		  <span class=\"code\">cubeNode</span>, \n               		  <span class=\"code\">lightNode</span>, and \n               		  <span class=\"code\">cameraNode</span> will be saved to the file, along with their node attributes. \n               		\n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_832F521525E348B69EB84AE4B17892F7\"></a> \n            <h2><a name=\"GUID-083FECD5-855C-466C-877E-A0D499D6786E\"></a>Node Attributes\n            </h2> \n            <p><a name=\"GUID-B14FDBCF-C23E-418B-A78A-472816D5682A\"></a>A mesh, a light, a camera, or any other object present in a scene is typically abstracted\n               by a subclass of \n               		  <span class=\"code\">FbxNodeAttribute</span>, for example \n               		  <span class=\"code\">FbxMesh</span>, \n               		  <span class=\"code\">FbxLight</span> or \n               		  <span class=\"code\">FbxCamera</span>. A \n               		  <span class=\"code\">FbxNodeAttribute</span> is bound to a \n               		  <span class=\"code\">FbxNode</span> to describe its position in the scene. This binding is performed using \n               		  <span class=\"code\">FbxNode::SetNodeAttribute()</span>. \n               		\n            </p> \n            <p><a name=\"GUID-E54DDF77-192F-412E-9BC2-DC2088AC5026\"></a>In the diagram above, the node attribute of \n               		  <span class=\"code\">FbxNode* lightNode</span> is a \n               		  <span class=\"code\">FbxLight</span>. The position of \n               		  <span class=\"code\">FbxLight* light</span> in the scene will therefore depend on the cumulative transformations applied to each\n               node from \n               		  <span class=\"code\">FbxNode* rootNode</span> to \n               		  <span class=\"code\">FbxNode* lightNode</span>. \n               		\n            </p> \n            <p><a name=\"GUID-81DABBC0-37FE-4269-8FCB-B2733BAF0AE6\"></a>More than one node attribute can be bound to a single node. This notion is useful\n               when dealing with meshes of varying levels of detail (LOD). Similarly, one node attribute\n               can be bound to multiple nodes. This technique is known as \"instancing\", and reduces\n               memory consumption. For more information, see \n               		  <span class=\"char_link\"><a href=\"GUID-0D483705-23D9-476D-A567-09609396B190.htm\">Instancing</a></span>. \n               		\n            </p> \n            <p><a name=\"GUID-F4963E30-1D0B-4742-80EA-EE522150345A\"></a> \n               <div><a name=\"GUID-865DC7D7-D9D0-4A43-92DF-81847E9174AD\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>A \n                     			 <span class=\"code\">FbxSurfaceMaterial</span> can also be bound to a node, and applied to its attributes to define, for example,\n                     the surface properties of a mesh. Observe that \n                     			 <span class=\"code\">FbxSurfaceMaterial</span> is \n                     			 <em class=\"strong\">not</em> a subclass of \n                     			 <span class=\"code\">FbxNodeAttribute</span>. \n                     		  \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_D539F208A9F5402AB4C3B1ED1D90D0BF\"></a> \n            <h2><a name=\"GUID-1E890135-6B2E-4D3F-812D-E7A987772F23\"></a>Transformation Data\n            </h2> \n            <p><a name=\"GUID-2160F957-73E5-4B64-9BA9-6890849393CD\"></a>The transformation data of a node includes its translation, rotation and scaling vectors\n               with respect to its parent. This data is represented as a set of \n               		  <span class=\"code\">FbxPropertyT</span> objects, accessible via \n               		  <span class=\"code\">FbxNode::LclTranslation</span>, \n               		  <span class=\"code\">FbxNode::LclRotation</span>, \n               		  <span class=\"code\">FbxNode::LclScaling</span>. Note that the \"<span class=\"code\">Lcl</span>\" prefix stands for \"local\". \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>FbxDouble3 translation = lNode-&gt;LclTranslation.Get();\nFbxDouble3 rotation = lNode-&gt;LclRotation.Get();\nFbxDouble3 scaling = lNode-&gt;LclScaling.Get();</pre></div> \n            <p><a name=\"GUID-FEDF1838-B2B2-487E-AE75-06FAA81DD8BB\"></a>The transformation data of a node can be limited by a \n               		  <span class=\"code\">FbxLimits</span> object, accessible via \n               		  <span class=\"code\">FbxNode::GetTranslationLimits()</span>, \n               		  <span class=\"code\">FbxNode::GetRotationLimits()</span>, and \n               		  <span class=\"code\">FbxNode::GetScalingLimits()</span>. Nodes can also be constrained using \n               		  <span class=\"code\">FbxConstraint</span> objects. Consult the class documentation of \n               		  <span class=\"code\">FbxLimits</span> and \n               		  <span class=\"code\">FbxConstraint</span> for more details. \n               		\n            </p> \n            <p><a name=\"GUID-D493F562-4A4F-4589-966D-8259CC5E92C4\"></a>The translation, rotation and scaling properties of a node with respect to the scene\'s\n               global coordinate system can be expressed as a transformation matrix. This transformation\n               matrix is obtained via \n               		  <span class=\"code\">FbxNode::EvaluateGlobalTransform()</span>. \n               		\n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_DF37099D050445F1BB6F96F65117C6DF\"></a> \n            <h2><a name=\"GUID-42C1E3E3-6190-4211-BE6E-A34623576AB6\"></a>Grouping Nodes\n            </h2> \n            <p><a name=\"GUID-0D952D95-0047-43CD-BE43-B00E7F15F49C\"></a>Instances of \n               		  <span class=\"code\">FbxNode</span> can exist without a bound \n               		  <span class=\"code\">FbxNodeAttribute</span>. In this case, such a \n               		  <span class=\"code\">FbxNode</span> can be used to group or position its children nodes in the scene. \n               		\n            </p> \n         </div><span class=\"noindex\">\n            <div class=\"topic-list\">\n               <h4 class=\"related\">Topics in this section</h4>\n               <ul class=\"jumplist\">\n                  <li class=\"topiclist-litem\"><a href=\"GUID-C35D98CB-5148-4B46-82D1-51077D8970EE.htm\">Transformation Data</a></li>\n                  <li class=\"topiclist-litem\"><a href=\"GUID-10CDD63C-79C1-4F2D-BB28-AD2BE65A02ED.htm\">Computing Transformation Matrices</a></li>\n               </ul>\n            </div></span><div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";