var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-2F4395F9-5BB3-413C-BCC4-C519DAF6A484\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Lights</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-2F4395F9-5BB3-413C-BCC4-C519DAF6A484\"></a><div class=\"head\">\n            <h1>Lights</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_1D7A645B353D4735B8027F8DF3BCDC3C\"></a> \n            <h2><a name=\"GUID-DAC71FF6-035B-4FD2-8547-E283C7872ACF\"></a>Creating a Light\n            </h2> \n            <p><a name=\"GUID-BE10D0EB-D783-4C0E-8CE5-A0973E5D7854\"></a>Lights in the FBX SDK are abstracted by the <span class=\"code\">FbxLight</span> class. A <span class=\"code\">FbxLight</span> is created like any other object in a scene. \n            </p> \n            <p><a name=\"GUID-8D4AB115-4FF3-46D2-A4BD-9D56218C56BB\"></a>By default, a <span class=\"code\">FbxLight</span> points along a node\'s <em class=\"strong\">negative Y axis</em>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a node for our light in the scene.\nFbxNode* lLightNode = FbxNode::Create(pScene, \"lightNode\");\n\n// Create a light.\nFbxLight* lLight = FbxLight::Create(pScene, \"light\");\n\n// Set the node attribute of the light node.\nlLightNode-&gt;SetNodeAttribute(lLight);\n\n// Add the light node to the root node in the scene.\nFbxNode* lRootNode = pScene-&gt;GetRootNode();\nlRootNode-&gt;AddChild(lLightNode);</pre></div> \n            <div><a name=\"GUID-D461ACE7-2A0B-4EDC-ACCF-4FF23EB5D686\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>A scene\'s ambient lighting is defined in its global settings, accessible via <span class=\"code\">FbxScene::GetGlobalSettings()</span>. Consult the <span class=\"code\">FbxGlobalSettings</span> class documentation for more information. \n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_1832C916800A43E3A50A561CB517AD0A\"></a> \n            <h2><a name=\"GUID-F8229CA0-081E-475F-BD5F-C8DA70C87F07\"></a>Light Type\n            </h2> \n            <p><a name=\"GUID-25B55B64-1131-4383-A161-4E9E77AE10A0\"></a>The behavior of the light can be defined by setting its <span class=\"code\">FbxLight::LightType</span> property. \n            </p> \n            <div class=\"codeBlock\"><pre>// Set the type of the light to a spotlight.\nlLight-&gt;LightType.Set(FbxLight::eSpot);</pre></div> \n            <p><a name=\"GUID-22BA4F16-9BB0-420F-9915-AB95BB77B529\"></a>The following table summarizes the behavior of each light type. \n            </p> <a name=\"WS1A9193826455F5FF-150B16DA11960D83164-6C23\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"33.33333333333333%\">\n                     <col width=\"66.66666666666666%\">\n                  </colgroup>\n                  <tr class=\"ruled-heading\">\n                     <th class=\"table-heading\">Light Type (<span class=\"code\">FbxLight::ELightType</span>) \n                     </th>\n                     <th class=\"table-heading\">Description </th>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::eSpot</span> \n                     </td>\n                     <td class=\"table-body\"> <a name=\"GUID-A410161E-19D2-48D5-9FD9-D39C008DB14E\"></a><p class=\"table-body\">Light spreads in a conical shape from its origin, like a spotlight. </p> <a name=\"GUID-5CF20574-D083-4BC0-9778-CF231458E625\"></a><p class=\"table-body\">The <span class=\"code\">FbxLight::InnerAngle</span> and <span class=\"code\">FbxLight::OuterAngle</span> properties determine the parameters of the cone in degrees. \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::ePoint</span> \n                     </td>\n                     <td class=\"table-body\">Light spreads uniformly in all directions from its origin. </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::eDirectional</span> \n                     </td>\n                     <td class=\"table-body\">Light spreads in a cylindrical shape from its origin. </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_319B419EE9C64EEDBFE585A72C3003CE\"></a> \n            <h2><a name=\"GUID-72D7D34A-B4DD-4DF2-A5A2-6EBBEAD18567\"></a>Pointing a Light\n            </h2> \n            <p><a name=\"GUID-F980BA12-6B75-4F9D-A7F3-863A2A72501B\"></a> A spotlight or a directional light can be forced to consistently point towards a\n               specific target in the scene. To do this, the light\'s node must have its target set\n               using <span class=\"code\">FbxNode::SetTarget()</span>. The <span class=\"code\">FbxMarker</span> node attribute is used in the target node. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a node to contain the marker. This will be our target node.\nFbxNode* lTargetNode = FbxNode::Create(pScene, \"targetNode\");\n\n// Create a marker node attribute.\nFbxMarker* lMarker = FbxMarker::Create(pScene, \"lightMarker\");\n\n// Set the marker as the target node\'s attribute.\nlTargetNode-&gt;SetNodeAttribute(lMarker);\n\n// Set our light node\'s target.\nlLightNode-&gt;SetTarget(lTargetNode); </pre></div> \n            <p><a name=\"GUID-DDFDC854-A29C-4C77-9E04-D0282EFBA73A\"></a>By default, a <span class=\"code\">FbxNode</span> uses its positive X axis as the aiming constraint. Recall that a newly created light\n               points along the node\'s negative Y axis by default. To make the light point along\n               the node\'s positive X axis, a rotation offset of 90 degrees must be applied to the\n               light\'s node using <span class=\"code\">FbxNode::SetPostTargetRotation()</span>. Consult the \"<em class=\"strong\">Node Target Management</em>\" section in the <span class=\"code\">FbxNode</span> class documentation for more information. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_2C7213DB78A94988ABD81D490C561EAB\"></a> \n            <h2><a name=\"GUID-92FECEE3-315D-4F60-BC52-F845BAD07AD0\"></a>Color and Intensity\n            </h2> \n            <p><a name=\"GUID-F6FFE451-4D07-4D3B-9507-259EBFB62516\"></a>A light\'s color is defined in its <span class=\"code\">FbxLight::Color</span> property. The default RGB value of a light is <span class=\"code\">(1.0, 1.0, 1.0)</span>, represented as a <span class=\"code\">FbxDouble3</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Set the light\'s color to (0, 1, 0.5).\nlLight-&gt;Color.Set(FbxDouble3(0.0, 1.0, 0.5));</pre></div> \n            <p><a name=\"GUID-D83BB877-F988-4DB8-96C2-D187700238DC\"></a>The intensity of a light is defined in its <span class=\"code\">FbxLight::Intensity</span> property. The default value of the intensity is <span class=\"code\">100.0</span>, represented as a <span class=\"code\">FbxDouble1</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Set the light\'s intensity to 50.0\nlLight-&gt;Intensity.Set(50.0)</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_30E2F0DFCA5541D3866F20D8F88D1379\"></a> \n            <h2><a name=\"GUID-7696874B-17CD-4B0D-8628-49CF02D482C0\"></a>Light Decay\n            </h2> \n            <p><a name=\"GUID-C40908D8-41AD-4EFB-AC5A-337D5771D201\"></a>The decay type of the light is defined in its <span class=\"code\">FbxLight::DecayType</span> property. \n            </p> \n            <div class=\"codeBlock\"><pre>// Set the decay type of the light to quadratic decay.\nlLight-&gt;DecayType.Set(FbxLight::eQuadratic);</pre></div> \n            <p><a name=\"GUID-626E978D-137F-43F1-825D-288DBDE5A281\"></a>The following table summarizes the decay types available. \n            </p> <a name=\"TABLE_249B0DADAA6F42BB87DED4714DE05A11\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"33.33333333333333%\">\n                     <col width=\"66.66666666666666%\">\n                  </colgroup>\n                  <tr class=\"ruled-heading\">\n                     <th class=\"table-heading\">Decay Type (<span class=\"code\">FbxLight::EDecayType</span>) \n                     </th>\n                     <th class=\"table-heading\">Description </th>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::eNone</span> \n                     </td>\n                     <td class=\"table-body\">No decay. The light\'s intensity will not diminish with distance. </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::eLinear</span> \n                     </td>\n                     <td class=\"table-body\">Linear decay. The light\'s intensity will diminish linearly with the distance from\n                        the light. \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::eQuadratic</span> \n                     </td>\n                     <td class=\"table-body\">Quadratic decay. The light\'s intensity will diminish with the squared distance from\n                        the light. This is the most physically accurate decay rate. \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight::eCubic</span> \n                     </td>\n                     <td class=\"table-body\">Cubic decay. The light\'s intensity will diminish with the cubed distance from the\n                        light. \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"GUID-537F64FD-6EB5-475A-8647-665BA6FE4E6D\"></a> \n               <div><a name=\"GUID-3C785C43-6998-44B3-862B-953660881C00\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>Other distance-based attenuation properties are available such as <span class=\"code\">FbxLight::EnableNearAttenuation</span> and <span class=\"code\">FbxLight::EnableFarAttenuation</span>. Consult the <span class=\"code\">FbxLight</span> class documentation for more information. \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_681A5245C0C74306B6A387AC47A3B83F\"></a> \n            <h2><a name=\"GUID-6174C9AB-70BD-48D5-B81A-44F39AC1FC62\"></a>Shadows\n            </h2> \n            <p><a name=\"GUID-683FFA63-5F2D-42C2-91ED-D90823E63763\"></a>Shadows are enabled using the <span class=\"code\">FbxLight::CastShadows</span> boolean property. The color of the light\'s shadow is defined in the <span class=\"code\">FbxLight::ShadowColor</span> property. The default RGB value of a light\'s shadow is <span class=\"code\">(0.0, 0.0, 0.0)</span>, represented as a <span class=\"code\">FbxDouble3</span>. A shadow texture may also be applied using <span class=\"code\">FbxLight::SetShadowTexture()</span>. \n            </p> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";