var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-4E8F95CA-CD9B-4E24-B8AD-7C399EA78137\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Exporting a Scene</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-4E8F95CA-CD9B-4E24-B8AD-7C399EA78137\"></a><div class=\"head\">\n            <h1>Exporting a Scene</h1>\n         </div>\n         <p><a name=\"GUID-C9403046-4A23-4B44-8E5A-92D9634049F8\"></a>The FBX SDK can export a scene into various file formats (see <span class=\"char_link\"><a href=\"GUID-0B122E01-7DB8-48E3-AADA-5E85A197FEE1.htm\">Supported File Formats</a></span>), including both <em class=\"mild\">binary</em> and ASCII FBX file formats. Note that only the binary FBX file format supports embedded\n            media (see <span class=\"char_link\"><a href=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17.htm\">I/O Settings</a></span>). \n         </p>\n         <div class=\"section\"><a name=\"SECTION_401FC03CAFE64E3CA7B96EB06E7CFC2F\"></a> \n            <h2><a name=\"GUID-4A9A63EB-44CF-407E-B03F-3FF0C0C14C23\"></a>Initializing the Exporter\n            </h2> \n            <p><a name=\"GUID-AD05B43C-DBDC-4D9E-A3F9-FCCCD422A97B\"></a>The scene exporting functionality of the FBX SDK is abstracted by the <span class=\"code\">FbxExporter</span> class. The steps required to initialize the exporter object and to export the scene\n               are analogous to the steps detailed in <span class=\"char_link\"><a href=\"GUID-ABFC58F1-C10D-42AE-A204-D79EB1D84B24.htm\">Importing a Scene</a></span>. A <span class=\"code\">FbxExporter</span> object is created using a reference to the program\'s <span class=\"code\">FbxManager</span> singleton. The <span class=\"code\">FbxExporter::Initialize()</span> method determines the filename and configuration settings of the exporter. This method\n               requires three parameters: <a name=\"OL_427C86B0D9D64AD69894C37A722FC110\"></a><ol type=\"1\" start=\"1\">\n                  <li>The path and filename of the file to which the scene will be exported. </li>\n                  <li>The numeric file format identifier. Typically, this parameter is set to <span class=\"code\">-1</span> to let the exporter automatically detect the file format according to the provided\n                     filename\'s extension. \n                  </li>\n                  <li>The <span class=\"code\">FbxIOSettings</span> object containing the export configuration options. See <span class=\"char_link\"><a href=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17.htm\">I/O Settings</a></span> for more information. \n                  </li>\n               </ol> \n            </p> \n            <div class=\"codeBlock\"><pre>#include &lt;fbxsdk.h&gt;\n#include &lt;fbxfilesdk/fbxio/fbxiosettings.h&gt;\n\n// ...\n\n// Create the FBX SDK manager\nFbxManager* lSdkManager = FbxManager::Create();\n\n// Create an IOSettings object.\nFbxIOSettings * ios = FbxIOSettings::Create(lSdkManager, IOSROOT );\nlSdkManager-&gt;SetIOSettings(ios);\n\n// ... Configure the FbxIOSettings object here ...\n\n// Create an exporter.\nFbxExporter* lExporter = FbxExporter::Create(lSdkManager, \"\");\n\n// Declare the path and filename of the file to which the scene will be exported.\n// In this case, the file will be in the same directory as the executable.\nconst char* lFilename = \"file.fbx\";\n\n// Initialize the exporter.\nbool lExportStatus lExporter-&gt;Initialize(lFilename, -1, lSdkManager-&gt;GetIOSettings());</pre></div> \n            <p><a name=\"GUID-9DF860E3-0765-4E7E-9D20-C6C927CE88F0\"></a>If any errors occur in the call to <span class=\"code\">FbxExporter::Initialize()</span>, the method returns <span class=\"code\">false</span>. To retrieve the error, you must call <span class=\"code\">GetStatus().GetErrorString()</span> from the <span class=\"code\">FbxExporter</span> object. For more information on error handling, see <span class=\"char_link\"><a href=\"GUID-5509751F-8679-4D32-987D-8343FD8F6D1A.htm\">Error Handling</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>if(!lExportStatus) {\n    printf(\"Call to FbxExporter::Initialize() failed.\\n\");\n    printf(\"Error returned: %s\\n\\n\", lExporter-&gt;GetStatus().GetErrorString());\n    return false;\n}</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_1AF3CFAE0F43472B8D09D7471B62AE1B\"></a> \n            <h2><a name=\"GUID-8176A325-454D-4345-80A0-B6531658671D\"></a>Exporting a Scene\n            </h2> \n            <p><a name=\"GUID-7F299290-808E-4FCC-A5E0-4D2D451E143F\"></a>The <span class=\"code\">FbxExporter::Export()</span> method exports the specified scene (<span class=\"code\">FbxScene</span>) to the file . For more information on building and manipulating scenes with the\n               FBX SDK, see <span class=\"char_link\"><a href=\"GUID-F194000D-5AD4-49C1-86CC-5DAC2CE64E97.htm\">Nodes and the Scene Graph</a></span>. \n            </p> \n            <p><a name=\"GUID-2B260E63-0A6F-4DF5-AE98-B23EE87593FD\"></a> \n               <div><a name=\"GUID-52E8A91F-F8A7-408F-8281-292C811CF5BE\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>During an export operation, the <em class=\"strong\">root node</em> of a scene is <em class=\"strong\">not exported</em> to the file. Only the children of the root node are exported to the file. As such,\n                     it is not recommended to associate anything to the root node that should be saved\n                     to a file. \n                  </div>\n               </div> \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a new scene so it can be populated by the imported file.\nFbxScene* lScene = FbxScene::Create(lSdkManager,\"myScene\");\n\n// ... Import a scene, or build a new one ...\n\n// Export the scene to the file.\nlExporter-&gt;Export(lScene);</pre></div> \n            <p><a name=\"GUID-A426D097-C562-493F-B048-D00F5E654527\"></a>Once the scene has been exported, the exporter can safely be destroyed to minimize\n               memory allocation. \n            </p> \n            <div class=\"codeBlock\"><pre>// Destroy the exporter.\nlExporter-&gt;Destroy();</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_2EFADC1CE2424FF6A9F40DF59481428C\"></a> \n            <h2><a name=\"GUID-5E742BA6-BF84-4E52-B621-E5A70C4EC569\"></a> File Version Number\n            </h2> \n            <p><a name=\"GUID-9312737A-8EB2-4AD2-BF89-EC3B9F403092\"></a>Use the <span class=\"code\">FbxManager::GetFileFormatVersion()</span> method to obtain the exported FBX file version number. For the latest FBX file version\n               number, see <span class=\"char_link\"><a href=\"GUID-0B122E01-7DB8-48E3-AADA-5E85A197FEE1.htm\">Supported File Formats</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// File format version numbers to be populated.\nint lMajor, lMinor, lRevision;\n\n// Populate the exported file format version numbers.\nFbxManager::GetFileFormatVersion(lMajor, lMinor, lRevision);</pre></div> \n            <p><a name=\"GUID-8C46FD70-D561-468D-9B28-9C54FC6E3DC5\"></a>As of FBX SDK 2012.0, the header file <span class=\"filePath\">fbxfilesdk_version.h</span> defines the preprocessor identifer <span class=\"code\">FBXSDK_VERSION_STRING</span>, which represents the version information as a string. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_B81B4BF3CCD54E1B87C38835A6C9F5F0\"></a> \n            <h2><a name=\"GUID-12F17E28-6EB2-4C01-8F71-9E49593CD5F3\"></a>Exporting With Embedded Media\n            </h2> \n            <p><a name=\"GUID-7E7C5579-59C3-43A6-9781-1498AD77B2FD\"></a>To export a scene and have its media embedded within the exported FBX file, the <em class=\"mild\">binary</em> (or <em class=\"mild\">native</em>) FBX file format identifier must be specified in the call to <span class=\"code\">FbxExporter::Initialize()</span>. The following code snippet illustrates how to perform this task. Recall that the\n               <span class=\"code\">FbxIOSettings</span> object must also be configured to embed the scene\'s media into the file (see <span class=\"char_link\"><a href=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17.htm\">I/O Settings</a></span>). \n            </p> \n            <div class=\"codeBlock\"><pre>// ... Initialize the required objects and variables ...\n\n// Set the FbxIOSettings EXP_FBX_EMBEDDED property to true.\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_EMBEDDED, true);\n\n// Get the appropriate file format.\nint lFileFormat = lSdkManager-&gt;GetIOPluginRegistry()-&gt;GetNativeWriterFormat();\n\n// Initialize the exporter to embed the scene\'s media into the exported file.\nbool lExportStatus lExporter-&gt;Initialize(lFilename, lFileFormat, lSdkManager-&gt;GetIOSettings());\n\n// ... Export the scene ...</pre></div> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li><span class=\"char_link\"><a href=\"GUID-75CD0DC4-05C8-4497-AC6E-EA11406EAE26.htm\">Customizing File Formats with FBX SDK I/O Plug-ins </a></span></li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";