var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-5509751F-8679-4D32-987D-8343FD8F6D1A\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Error Handling</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-5509751F-8679-4D32-987D-8343FD8F6D1A\"></a><div class=\"head\">\n            <h1>Error Handling</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_02BEDAA2E35743789C90A3005620D930\"></a> \n            <h2><a name=\"GUID-6AFBC857-8E38-44E9-AA81-69FA5DAD558F\"></a>Errors\n            </h2> \n            <p><a name=\"GUID-97D46743-0088-4BAF-8EC8-DA4972ACDA83\"></a>Many member functions of FBX classes may trigger an error. In such a case: \n            </p> <a name=\"WS1A9193826455F5FF-E569A012180CE5891-1ECB\"></a><ul>\n               <li>The function returns <span class=\"code\">false</span>. \n               </li>\n               <li> <span class=\"code\"> <em class=\"mild\">objectname</em>-&gt;GetStatus().GetErrorString()</span> returns a string with an error message. \n               </li>\n               <li> <span class=\"code\"> <em class=\"mild\">objectname</em>-&gt;GetStatus().GetCode()</span> retrieves the type of error that occurred as specified in the enumeration. \n               </li>\n            </ul> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_AC813D693E234142BAC675B0881DECCD\"></a> \n            <h2><a name=\"GUID-84F8BFEC-292E-42D4-AB01-7377AA974B9C\"></a>Error Handling Sample\n            </h2> \n            <p><a name=\"WS1A9193826455F5FF-E569A012180CE5891-1EC7\"></a>The following code appears in <span class=\"code\">Common/Common.cxx</span>. It illustrates the error-handling procedure for a file import operation, including\n               password protection. Some code has been replaced by \"<span class=\"code\">// ...</span>\" to emphasize the error-handling functionality. \n            </p> \n            <div class=\"codeBlock\"><pre>bool LoadScene(FbxManager* pManager, FbxDocument* pScene, const char* pFilename)\n{\n  //…\n\n  bool lStatus;\n  char lPassword[1024];\n\n  //…\n  \n  // Create an importer.\n  FbxImporter* lImporter = FbxImporter::Create(pManager,\"\");\n    \n  // Initialize the importer by providing a filename.\n  const bool lImportStatus = lImporter-&gt;Initialize(pFilename, -1, pManager-&gt;GetIOSettings());\n  \n  // …\n\n  if(!lImportStatus)\n    {\n        FbxString error = lImporter-&gt;GetStatus().GetErrorString();\n        FBXSDK_printf(\"Call to FbxImporter::Initialize() failed.\\n\");\n        FBXSDK_printf(\"Error returned: %s\\n\\n\", error.Buffer());\n\n        if (lImporter-&gt;GetStatus().GetCode() == FbxStatus::eInvalidFileVersion)\n        {\n          FBXSDK_printf(\"FBX file format version for this FBX SDK is %d.%d.%d\\n\", lSDKMajor, lSDKMinor, lSDKRevision);\n          FBXSDK_printf(\"FBX file format version for file \'%s\' is %d.%d.%d\\n\\n\", pFilename, lFileMajor, lFileMinor, lFileRevision);\n        }\n        \n        return false;\n    }\n    // …\n    // Import the scene.\n    \n    lStatus = lImporter-&gt;Import(pScene);\n    \n    if(lStatus == false &amp;&amp; lImporter-&gt;GetStatus().GetCode() == FbxStatus::ePasswordError)\n    {\n        FBXSDK_printf(\"Please enter password: \");\n        lPassword[0] = \'\\0\';\n        \n        FBXSDK_CRT_SECURE_NO_WARNING_BEGIN\n        scanf(\"%s\", lPassword);\n        FBXSDK_CRT_SECURE_NO_WARNING_END\n        \n        FbxString lString(lPassword);\n        \n        IOS_REF.SetStringProp(IMP_FBX_PASSWORD, lString);\n        IOS_REF.SetBoolProp(IMP_FBX_PASSWORD_ENABLE, true);\n        \n        lStatus = lImporter-&gt;Import(pScene);\n        \n        if(lStatus == false &amp;&amp; lImporter-&gt;GetStatus().GetCode() == FbxStatus::ePasswordError)\n        {\n          FBXSDK_printf(\"\\nPassword is wrong, import aborted.\\n\");\n        }\n    }\n    \n    // Destroy the importer.\n    lImporter-&gt;Destroy();\n    \n    return lStatus;\n}\n</pre></div> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";