var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-AD69D141-C5DD-4609-AD5A-4805321FADD4\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Textures</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-AD69D141-C5DD-4609-AD5A-4805321FADD4\"></a><div class=\"head\">\n            <h1>Textures</h1>\n         </div>\n         <div><a name=\"GUID-3EF2359A-B783-41F8-859C-2995ACDAE772\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The sample code in this topic is also presented in the <span class=\"code\">ExportScene03/main.cxx</span> sample program. \n            </div>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_954782C09A344EDC8D29409E40F6B232\"></a> \n            <h2><a name=\"GUID-A15166D6-4E88-488D-902E-BE9689452423\"></a>Creating a Texture From a File\n            </h2> \n            <p><a name=\"GUID-5C79D541-5221-44BA-B7B5-FB6A40EF76EB\"></a><span class=\"code\">FbxTexture</span> is the base class for textures in the FBX SDK. Textures rely on an underlying material\n               to influence how the geometry is rendered. In the following code sample, we associate\n               the <em class=\"strong\">diffuse</em>, <em class=\"strong\">ambient</em> and <em class=\"strong\">emissive</em> channels of a Phong material to three distinct instances of <span class=\"code\">FbxFileTexture</span>. <span class=\"code\">FbxFileTexture</span> represents any texture loaded from a file. For more information on referencing embedded\n               or non-embedded media in files, see <span class=\"char_link\"><a href=\"GUID-6766AFC0-4F25-4436-A5EE-BCDAFC6CB583.htm\">Referencing Media</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create texture for cube.\nvoid CreateTexture(FbxScene* pScene, FbxMesh* pMesh)\n{\n    // A texture need to be connected to a property on the material,\n    // so let\'s use the material (if it exists) or create a new one\n    FbxSurfacePhong* lMaterial = NULL;\n\n    //get the node of mesh, add material for it.\n    FbxNode* lNode = pMesh-&gt;GetNode();\n    if(lNode)\n    {\n        lMaterial = lNode-&gt;GetSrcObject&lt;FbxSurfacePhong&gt;(0);\n        if (lMaterial == NULL)\n        {\n            FbxString lMaterialName = \"toto\";\n            FbxString lShadingName  = \"Phong\";\n            FbxDouble3 lBlack(0.0, 0.0, 0.0);\n            FbxDouble3 lRed(1.0, 0.0, 0.0);\n            FbxDouble3 lDiffuseColor(0.75, 0.75, 0.0);\n            lMaterial = FbxSurfacePhong::Create(pScene, lMaterialName.Buffer());\n\n            // Generate primary and secondary colors.\n            lMaterial-&gt;Emissive           .Set(lBlack);\n            lMaterial-&gt;Ambient            .Set(lRed);\n            lMaterial-&gt;AmbientFactor      .Set(1.);\n            // Add texture for diffuse channel\n            lMaterial-&gt;Diffuse           .Set(lDiffuseColor);\n            lMaterial-&gt;DiffuseFactor     .Set(1.);\n            lMaterial-&gt;TransparencyFactor.Set(0.4);\n            lMaterial-&gt;ShadingModel      .Set(lShadingName);\n            lMaterial-&gt;Shininess         .Set(0.5);\n            lMaterial-&gt;Specular          .Set(lBlack);\n            lMaterial-&gt;SpecularFactor    .Set(0.3);\n\n            lNode-&gt;AddMaterial(lMaterial);\n        }\n    }\n\n    FbxFileTexture* lTexture = FbxFileTexture::Create(pScene,\"Diffuse Texture\");\n\n    // Set texture properties.\n    lTexture-&gt;SetFileName(\"scene03.jpg\"); // Resource file is in current directory.\n    lTexture-&gt;SetTextureUse(FbxTexture::eStandard);\n    lTexture-&gt;SetMappingType(FbxTexture::eUV);\n    lTexture-&gt;SetMaterialUse(FbxFileTexture::eModelMaterial);\n    lTexture-&gt;SetSwapUV(false);\n    lTexture-&gt;SetTranslation(0.0, 0.0);\n    lTexture-&gt;SetScale(1.0, 1.0);\n    lTexture-&gt;SetRotation(0.0, 0.0);\n\n    // don\'t forget to connect the texture to the corresponding property of the material\n    if (lMaterial)\n        lMaterial-&gt;Diffuse.ConnectSrcObject(lTexture);\n\n    lTexture = FbxFileTexture::Create(pScene,\"Ambient Texture\");\n\n    // Set texture properties.\n    lTexture-&gt;SetFileName(\"gradient.jpg\"); // Resource file is in current directory.\n    lTexture-&gt;SetTextureUse(FbxTexture::eStandard);\n    lTexture-&gt;SetMappingType(FbxTexture::eUV);\n    lTexture-&gt;SetMaterialUse(FbxFileTexture::eModelMaterial);\n    lTexture-&gt;SetSwapUV(false);\n    lTexture-&gt;SetTranslation(0.0, 0.0);\n    lTexture-&gt;SetScale(1.0, 1.0);\n    lTexture-&gt;SetRotation(0.0, 0.0);\n\n    // don\'t forget to connect the texture to the corresponding property of the material\n    if (lMaterial)\n        lMaterial-&gt;Ambient.ConnectSrcObject(lTexture);\n\n    lTexture = FbxFileTexture::Create(pScene,\"Emissive Texture\");\n\n    // Set texture properties.\n    lTexture-&gt;SetFileName(\"spotty.jpg\"); // Resource file is in current directory.\n    lTexture-&gt;SetTextureUse(FbxTexture::eStandard);\n    lTexture-&gt;SetMappingType(FbxTexture::eUV);\n    lTexture-&gt;SetMaterialUse(FbxFileTexture::eModelMaterial);\n    lTexture-&gt;SetSwapUV(false);\n    lTexture-&gt;SetTranslation(0.0, 0.0);\n    lTexture-&gt;SetScale(1.0, 1.0);\n    lTexture-&gt;SetRotation(0.0, 0.0);\n\n    // don\'t forget to connect the texture to the corresponding property of the material\n    if (lMaterial)\n        lMaterial-&gt;Emissive.ConnectSrcObject(lTexture);\n}</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_C6B60D1589DB4BA2B73E5D8197577F34\"></a> \n            <h2><a name=\"GUID-B270D4F4-787A-49F9-849E-8B9032308CBE\"></a>Example: Creating a Textured Cube\n            </h2> \n            <p><a name=\"GUID-A89C2683-3304-4AB7-BC94-938DDC2FC307\"></a>The following code sample makes use of the <span class=\"code\">CreateTexture()</span> function we defined above. We first begin by defining the control points and normals\n               of the cube. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a cube with a texture. \nFbxNode* CreateCubeWithTexture(FbxScene* pScene, char* pName)\n{\n    int i, j;\n    FbxMesh* lMesh = FbxMesh::Create(pScene,pName);\n\n    FbxVector4 vertex0(-50, 0, 50);\n    FbxVector4 vertex1(50, 0, 50);\n    FbxVector4 vertex2(50, 100, 50);\n    FbxVector4 vertex3(-50, 100, 50);\n    FbxVector4 vertex4(-50, 0, -50);\n    FbxVector4 vertex5(50, 0, -50);\n    FbxVector4 vertex6(50, 100, -50);\n    FbxVector4 vertex7(-50, 100, -50);\n\n    FbxVector4 lNormalXPos(1, 0, 0);\n    FbxVector4 lNormalXNeg(-1, 0, 0);\n    FbxVector4 lNormalYPos(0, 1, 0);\n    FbxVector4 lNormalYNeg(0, -1, 0);\n    FbxVector4 lNormalZPos(0, 0, 1);\n    FbxVector4 lNormalZNeg(0, 0, -1);\n\n    // Create control points.\n    lMesh-&gt;InitControlPoints(24);\n    FbxVector4* lControlPoints = lMesh-&gt;GetControlPoints();\n\n    lControlPoints[0] = vertex0;\n    lControlPoints[1] = vertex1;\n    lControlPoints[2] = vertex2;\n    lControlPoints[3] = vertex3;\n    lControlPoints[4] = vertex1;\n    lControlPoints[5] = vertex5;\n    lControlPoints[6] = vertex6;\n    lControlPoints[7] = vertex2;\n    lControlPoints[8] = vertex5;\n    lControlPoints[9] = vertex4;\n    lControlPoints[10] = vertex7;\n    lControlPoints[11] = vertex6;\n    lControlPoints[12] = vertex4;\n    lControlPoints[13] = vertex0;\n    lControlPoints[14] = vertex3;\n    lControlPoints[15] = vertex7;\n    lControlPoints[16] = vertex3;\n    lControlPoints[17] = vertex2;\n    lControlPoints[18] = vertex6;\n    lControlPoints[19] = vertex7;\n    lControlPoints[20] = vertex1;\n    lControlPoints[21] = vertex0;\n    lControlPoints[22] = vertex4;\n    lControlPoints[23] = vertex5;\n\n\n    // We want to have one normal for each vertex (or control point),\n    // so we set the mapping mode to eByControlPoint.\n    FbxGeometryElementNormal* lGeometryElementNormal= lMesh-&gt;CreateElementNormal();\n\n    lGeometryElementNormal-&gt;SetMappingMode(FbxGeometryElement::eByControlPoint);\n\n    // Here are two different ways to set the normal values.\n    bool firstWayNormalCalculations=true;\n    if (firstWayNormalCalculations)\n    {    \n        // The first method is to set the actual normal value\n        // for every control point.\n        lGeometryElementNormal-&gt;SetReferenceMode(FbxGeometryElement::eDirect);\n\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\n    }\n    else\n    {\n        // The second method is to the possible values of the normals\n        // in the direct array, and set the index of that value\n        // in the index array for every control point.\n        lGeometryElementNormal-&gt;SetReferenceMode(FbxGeometryElement::eIndexToDirect);\n\n        // Add the 6 different normals to the direct array\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\n        lGeometryElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\n\n        // Now for each control point, we need to specify which normal to use\n        lGeometryElementNormal-&gt;GetIndexArray().Add(0); // index of lNormalZPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(0); // index of lNormalZPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(0); // index of lNormalZPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(0); // index of lNormalZPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(1); // index of lNormalXPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(1); // index of lNormalXPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(1); // index of lNormalXPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(1); // index of lNormalXPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(2); // index of lNormalZNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(2); // index of lNormalZNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(2); // index of lNormalZNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(2); // index of lNormalZNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(3); // index of lNormalXNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(3); // index of lNormalXNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(3); // index of lNormalXNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(3); // index of lNormalXNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(4); // index of lNormalYPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(4); // index of lNormalYPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(4); // index of lNormalYPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(4); // index of lNormalYPos in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(5); // index of lNormalYNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(5); // index of lNormalYNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(5); // index of lNormalYNeg in the direct array.\n        lGeometryElementNormal-&gt;GetIndexArray().Add(5); // index of lNormalYNeg in the direct array.\n    }\n</pre></div> \n            <p><a name=\"GUID-BF709D5D-E8C3-43B5-A72A-34BFE999EBB7\"></a>We then call <span class=\"code\">FbxMesh::CreateElementUV()</span> to create three new texture layers in the mesh - one for each of the <em class=\"strong\">diffuse</em>, <em class=\"strong\">ambient</em>, and <em class=\"strong\">emissive</em> material channels. The resulting texture layer elements (<span class=\"code\">FbxGeometryElementUV</span>) define how to map the texture\'s UV coordinates to each of the polygon\'s vertices.\n               Lastly, we call <span class=\"code\">FbxNode::SetShadingMode()</span> with the value <span class=\"code\">FbxNode::eTextureShading</span> to allow our textures to be rendered in the scene. \n            </p> \n            <div class=\"codeBlock\"><pre>    // Array of polygon vertices.\n    int lPolygonVertices[] = { 0, 1, 2, 3,\n        4, 5, 6, 7,\n        8, 9, 10, 11,\n        12, 13, 14, 15,\n        16, 17, 18, 19,\n        20, 21, 22, 23 };\n\n    // Create UV for Diffuse channel\n    FbxGeometryElementUV* lUVDiffuseElement = lMesh-&gt;CreateElementUV( \"DiffuseUV\");\n    K_ASSERT( lUVDiffuseElement != NULL);\n    lUVDiffuseElement-&gt;SetMappingMode(FbxGeometryElement::eByPolygonVertex);\n    lUVDiffuseElement-&gt;SetReferenceMode(FbxGeometryElement::eIndexToDirect);\n\n    FbxVector2 lVectors0(0, 0);\n    FbxVector2 lVectors1(1, 0);\n    FbxVector2 lVectors2(1, 1);\n    FbxVector2 lVectors3(0, 1);\n\n    lUVDiffuseElement-&gt;GetDirectArray().Add(lVectors0);\n    lUVDiffuseElement-&gt;GetDirectArray().Add(lVectors1);\n    lUVDiffuseElement-&gt;GetDirectArray().Add(lVectors2);\n    lUVDiffuseElement-&gt;GetDirectArray().Add(lVectors3);\n\n\n    // Create UV for Ambient channel\n    FbxGeometryElementUV* lUVAmbientElement = lMesh-&gt;CreateElementUV(\"AmbientUV\");\n\n    lUVAmbientElement-&gt;SetMappingMode(FbxGeometryElement::eByPolygonVertex);\n    lUVAmbientElement-&gt;SetReferenceMode(FbxGeometryElement::eIndexToDirect);\n\n    lVectors0.Set(0, 0);\n    lVectors1.Set(1, 0);\n    lVectors2.Set(0, 0.418586879968643);\n    lVectors3.Set(1, 0.418586879968643);\n\n    lUVAmbientElement-&gt;GetDirectArray().Add(lVectors0);\n    lUVAmbientElement-&gt;GetDirectArray().Add(lVectors1);\n    lUVAmbientElement-&gt;GetDirectArray().Add(lVectors2);\n    lUVAmbientElement-&gt;GetDirectArray().Add(lVectors3);\n\n    // Create UV for Emissive channel\n    FbxGeometryElementUV* lUVEmissiveElement = lMesh-&gt;CreateElementUV(\"EmissiveUV\");\n\n    lUVEmissiveElement-&gt;SetMappingMode(FbxGeometryElement::eByPolygonVertex);\n    lUVEmissiveElement-&gt;SetReferenceMode(FbxGeometryElement::eIndexToDirect);\n\n    lVectors0.Set(0.2343, 0);\n    lVectors1.Set(1, 0.555);\n    lVectors2.Set(0.333, 0.999);\n    lVectors3.Set(0.555, 0.666);\n\n    lUVEmissiveElement-&gt;GetDirectArray().Add(lVectors0);\n    lUVEmissiveElement-&gt;GetDirectArray().Add(lVectors1);\n    lUVEmissiveElement-&gt;GetDirectArray().Add(lVectors2);\n    lUVEmissiveElement-&gt;GetDirectArray().Add(lVectors3);\n\n    //Now we have set the UVs as eIndexToDirect reference and in eByPolygonVertex mapping mode\n    //we must update the size of the index array.\n    lUVDiffuseElement-&gt;GetIndexArray().SetCount(24);\n    lUVAmbientElement-&gt;GetIndexArray().SetCount(24);\n    lUVEmissiveElement-&gt;GetIndexArray().SetCount(24);\n\n\n\n    // Create polygons. Assign texture and texture UV indices.\n    for(i = 0; i &lt; 6; i++)\n    {\n        //we won\'t use the default way of assigning textures, as we have\n        //textures on more than just the default (diffuse) channel.\n        lMesh-&gt;BeginPolygon(-1, -1, false);\n\n\n\n        for(j = 0; j &lt; 4; j++)\n        {\n            //this function points \n            lMesh-&gt;AddPolygon(lPolygonVertices[i*4 + j] // Control point index. \n            );\n            //Now we have to update the index array of the UVs for diffuse, ambient and emissive\n            lUVDiffuseElement-&gt;GetIndexArray().SetAt(i*4+j, j);\n            lUVAmbientElement-&gt;GetIndexArray().SetAt(i*4+j, j);\n            lUVEmissiveElement-&gt;GetIndexArray().SetAt(i*4+j, j);\n\n        }\n\n        lMesh-&gt;EndPolygon ();\n    }\n\n    FbxNode* lNode = FbxNode::Create(pScene,pName);\n\n    lNode-&gt;SetNodeAttribute(lMesh);\n    lNode-&gt;SetShadingMode(FbxNode::eTextureShading);\n\n    CreateTexture(pScene, lMesh);\n\n    return lNode;\n}</pre></div> \n         </div><span class=\"noindex\">\n            <div class=\"topic-list\">\n               <h4 class=\"related\">Topics in this section</h4>\n               <ul class=\"jumplist\">\n                  <li class=\"topiclist-litem\"><a href=\"GUID-452768B5-7A4F-45BD-AC37-DC1A034DCF3B.htm\">Layered Textures</a></li>\n               </ul>\n            </div></span><div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";