var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-B3BF2A41-2262-4706-ABB3-D3C861B3F03B\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Managing Memory with the FBX SDK Manager</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-B3BF2A41-2262-4706-ABB3-D3C861B3F03B\"></a><div class=\"head\">\n            <h1>Managing Memory with the FBX SDK Manager</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_391386F1D16A437EA23A46BBA6EDF954\"></a> \n            <h2><a name=\"GUID-06EB1FFA-B29A-46EB-AB03-741B6B404854\"></a>Creating an FBX SDK Manager\n            </h2> \n            <p><a name=\"GUID-31C7DEDA-C97D-4CA7-868A-423BFB51E267\"></a>The <span class=\"code\">FbxManager</span> class is responsible for creating, managing, and destroying FBX SDK objects. Only\n               one <span class=\"code\">FbxManager</span> instance is required for a given program. The creation of this <span class=\"code\">FbxManager</span> singleton is typically the first action of any FBX SDK program. \n            </p> \n            <div class=\"codeBlock\"><pre>FbxManager* lSdkManager = FbxManager::Create();</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_2580FB568D5F467DAE8CA8A933D79542\"></a> \n            <h2><a name=\"GUID-66A3BA9A-F970-4124-B30D-EA3AB9FE12EA\"></a>Creating Objects with the FBX SDK Manager\n            </h2> \n            <p><a name=\"GUID-C87405DF-8CB7-4E38-941D-ED829649CA62\"></a>Objects in the FBX SDK are created or destroyed by means of their respective <span class=\"code\">Create()</span> and <span class=\"code\">Destroy()</span> member functions. At a lower level, these functions rely on the <span class=\"code\">FbxManager</span> to allocate and release memory. In the following example, the <span class=\"code\">FbxManager</span> is passed as a parameter to the <span class=\"code\">FbxScene::Create()</span> function to instantiate a new scene. \n            </p> \n            <div class=\"codeBlock\"><pre>FbxScene* lScene = FbxScene::Create(lSdkManager, \"Scene Name\");</pre></div> \n            <div class=\"section\"><a name=\"SECTION_67B8E09D26F447E4986E6BF5A591A5D9\"></a> \n               <h3><a name=\"GUID-8F2DA2E0-B1EB-4BB0-9CBB-BCC5E75CEF35\"></a>Memory Allocation\n               </h3> \n               <p><a name=\"GUID-3D13BD20-68C2-444B-84F0-F2C6E956108B\"></a>The SDK object manager (<span class=\"code\">FbxManager</span>) automatically allocates a sufficient amount of memory to contain the new object.\n                  Though not necessary, the memory allocation strategy of the object manager can be\n                  customized using the handler setter functions such as <span class=\"code\">FbxSetMallocHandler</span> located in <span class=\"code\">fbxalloc.h</span>. See the <span class=\"filePath\">ViewScene/main.cxx</span> example for information about how to use a custom memory allocator. \n               </p> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_FFB818B21C0840998CA6DE5A53596CC6\"></a> \n               <h3><a name=\"GUID-4145F099-0AEC-4414-977C-106B49BAC8B4\"></a>Naming FBX Objects\n               </h3> \n               <p><a name=\"GUID-03D9C3BF-10A5-447F-ABDB-1727E8A3C6C3\"></a>In the code snippet above, the second parameter of <span class=\"code\">FbxScene::Create()</span> is a string. When objects are created, a string can be used to specify the name of\n                  the new object. This string does not have to be unique, and it is acceptable to pass\n                  an empty string <span class=\"code\">\"\"</span>. Object names can facilitate the debugging of FBX SDK applications and their outputs.\n                  \n               </p> \n            </div> \n            <p><a name=\"GUID-BB587DF4-A154-43E3-9513-0D1C3C878883\"></a> \n               <div><a name=\"GUID-BD44F77C-E05A-48E4-8FF4-F35D80B85FE1\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span><em class=\"strong\">The current implementation of the FBX SDK is not guaranteed to be thread-safe</em>. Objects which are created using the same <span class=\"code\">FbxManager</span>, but that are accessed in different threads will likely cause the application to\n                     crash. This is a known issue, and there are plans to ensure the thread-safety of the\n                     FBX SDK in a future release. \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_0DAB8F95F7A8474FBCF17278D71B1F2D\"></a> \n            <h2><a name=\"GUID-A82656D1-DAD6-4A94-B693-4AE434E39E9E\"></a>Creating Objects within a Scene\n            </h2> \n            <p><a name=\"GUID-538E4857-875F-4493-BC40-231C23E712CC\"></a>A <span class=\"code\">FbxScene</span> object can contain a variety of scene elements such as meshes, lights, animations,\n               characters, etc. These elements should be created with a reference to the scene in\n               which they exist. As such, when the scene is exported, so are all of its elements.\n               When the scene is destroyed, the memory allocated to all of its objects is also released.\n               The <span class=\"char_link\"><a href=\"GUID-F194000D-5AD4-49C1-86CC-5DAC2CE64E97.htm\">Nodes and the Scene Graph</a></span> section describes the use of the <span class=\"code\">FbxNode</span> and <span class=\"code\">FbxNodeAttribute</span> classes to define and manipulate scene elements. For now, we will only look at how\n               a <span class=\"code\">FbxScene</span> can be used to create these objects. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a node object\nFbxNode* lNode = FbxNode::Create(lScene, \"node\");\n&nbsp;\n// Create a mesh object\nFbxMesh* lMesh = FbxMesh::Create(lScene, \"\");</pre></div> \n            <p><a name=\"GUID-F4349094-2265-4ED9-AECE-D84FF971F763\"></a> \n               <div><a name=\"GUID-56470A60-CA4B-4065-A5F6-FDF3142BA467\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>It is possible to create scene elements (<span class=\"code\">FbxNode</span>, <span class=\"code\">FbxNodeAttribute</span>) using only a reference to the <span class=\"code\">FbxManager</span> (instead of <span class=\"code\">FbxScene</span>), however when the scene is destroyed, those scene elements will not be destroyed\n                     with it; they will only be destroyed either explicitly or when the <span class=\"code\">FbxManager</span> is destroyed. \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-191A86AC126720FA354-5CDC\"></a> \n            <h2><a name=\"GUID-C4CF4DA4-8A77-4364-89C9-218D95E483A7\"></a>Destroying objects\n            </h2> \n            <p><a name=\"GUID-475E5F60-E5C2-4F6A-941C-28F027BA6194\"></a>An FBX SDK object should be explicitly destroyed by calling its <span class=\"code\">Destroy()</span> member function. The <span class=\"code\">FbxManager</span> will automatically free the memory allocated for that object, and will update all\n               the internal <em class=\"mild\">connections</em> between that <em class=\"mild\">object</em> (<span class=\"code\">FbxObject</span>), its <em class=\"mild\">properties</em> (<span class=\"code\">FbxProperty</span>), and other <span class=\"code\">FbxObject</span> to remove any inconsistencies. For more information on the concept of <em class=\"mild\">connections</em> in the FBX SDK, see <span class=\"char_link\"><a href=\"GUID-BB63A93A-7663-4256-B060-8EA35CB0FF3A.htm\">Connections</a></span>. The following code snippet illustrates how to destroy the objects we instantiated\n               above. \n            </p> \n            <div class=\"codeBlock\"><pre>\n// Destroy these objects\nlMesh-&gt;Destroy();&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Destroy the mesh\nlNode-&gt;Destroy();&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Destroy the node\nlScene-&gt;Destroy();&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Destroy the scene and its objects\nlSDKManager-&gt;Destroy() // Destroy SDK Manager and any remaining objects which it manages.\n</pre></div> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";