var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-BB63A93A-7663-4256-B060-8EA35CB0FF3A\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Connections</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-BB63A93A-7663-4256-B060-8EA35CB0FF3A\"></a><div class=\"head\">\n            <h1>Connections</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_50CA5F9794C34D7EB6FDDC35B62140F3\"></a> \n            <h2><a name=\"GUID-B005101B-FC83-49CA-959F-31B5E7C81616\"></a>Visualizing Connections\n            </h2> \n            <p><a name=\"WS73099CC142F487551FEA285E1221E4F9FF8-7FD9\"></a>A \n               		  <em class=\"mild\">connection</em> is an FBX SDK data structure which manages the two-way relationship between FBX \n               		  <em class=\"strong\">objects</em> and/or FBX \n               		  <em class=\"strong\">properties</em>. To guarantee the consistency of connections within the FBX SDK, the actual datastructure\n               is not publically exposed. Instead, connections can be manipulated using the \n               		  <span class=\"code\">FbxObject</span> and \n               		  <span class=\"code\">FbxProperty</span> connection-management methods such as: \n               		  <span class=\"code\">FbxObject::ConnectSrcObject()</span>, \n               		  <span class=\"code\">FbxObject::ConnectDstObject()</span>, \n               		  <span class=\"code\">FbxProperty::ConnectDstObject()</span>, \n               		  <span class=\"code\">FbxProperty::ConnectSrcProperty()</span>, etc. \n               		\n            </p> \n            <p><a name=\"GUID-21E197E0-B7BB-4827-BA5E-4B69981681BB\"></a>Connections can be visualized as a \n               		  <em class=\"mild\">destination</em> and \n               		  <em class=\"mild\">source</em> hierarchy of FBX \n               		  <em class=\"strong\">objects</em> and \n               		  <em class=\"strong\">properties</em>. \n               		\n            </p> \n            <p><a name=\"GUID-F076C82C-9AD0-4C52-9B34-426960909750\"></a> <a name=\"UL_05B858C5450D42FA97F375C335E07219\"></a><ul>\n                  <li><em class=\"strong\">object-property connections:</em> Properties are contained within objects as \n                     			 <em class=\"mild\">sources</em>. Calling \n                     			 <span class=\"code\">FbxObject::GetSrcProperty()</span> will return the object\'s \n                     			 <em class=\"mild\">source</em> property at the given index. Symmetrically, calling \n                     			 <span class=\"code\">FbxProperty::GetDstObject()</span> will return the property\'s \n                     			 <em class=\"mild\">destination</em> object. \n                     		  \n                  </li>\n                  <li><em class=\"strong\">object-object connections:</em> Parent-child relationships among \n                     			 <em class=\"strong\">objects</em> use connections (for example: the node hierarchy of a scene). Typically, an object\'s\n                     children are \n                     			 <em class=\"mild\">sources</em>, and are accessed using \n                     			 <span class=\"code\">FbxObject::GetSrcObject()</span>. An object\'s parent is a \n                     			 <em class=\"mild\">destination</em>, and is accessed using \n                     			 <span class=\"code\">FbxObject::GetDstObject()</span>. \n                     		  \n                  </li>\n                  <li><em class=\"strong\">property-property connections:</em> Parent-child relationships among \n                     			 <em class=\"strong\">properties</em> also use connections (for example: the property hierarchy of \n                     			 <span class=\"code\">FbxIOSettings</span>). Typically, a property\'s children are \n                     			 <em class=\"mild\">sources</em>, and are accessed using \n                     			 <span class=\"code\">FbxProperty::GetSrcProperty()</span>. A property\'s parent is referred to as a \n                     			 <em class=\"mild\">destination</em>, and is accessed using \n                     			 <span class=\"code\">FbxProperty::GetDstProperty()</span>. \n                     		  \n                  </li>\n               </ul> \n            </p> \n            <p><a name=\"GUID-BC9C7B13-4900-4ACA-ADD5-F49B6B8839C4\"></a></p> <img src=\"../images/GUID-A06F2F6A-AC1D-455A-B1AF-2B78619F557F-low.png\"> \n            <p><a name=\"GUID-926C1BB9-7BF6-48BD-ACE7-A480538C6668\"></a> <a name=\"UL_CD9188BFC5434A3287480AC2C6D6BAB0\"></a><p class=\"list-intro\"><a name=\"GUID-974FD201-A927-4389-845A-5DE1C62EEC8D\"></a>The \n                  		  <em class=\"mild\">destination</em> and \n                  		  <em class=\"mild\">source</em> connections in the diagram above are illustrated in the following code samples.\n               </p>\n               <ul>\n                  <li>Accessing the \n                     			 <em class=\"strong\">source objects</em> of \n                     			 <span class=\"code\">obj0</span>: \n                     			 \n                     <div class=\"codeBlock\"><pre>// ... Assume obj0 has been initialized to reflect the diagram above...\n// For illustrative purposes, count the number of source objects connected to obj0.\nint numSrcObjects = obj0-&gt;GetSrcObjectCount(); // numSrcObjects = 2\n\n// Access the two source objects connected to obj0\n// Note that obj0-&gt;GetSrcObject(0) is equivalent to calling obj0-&gt;GetSrcObject()\nFbxObject* obj1 = obj0-&gt;GetSrcObject(0);\nFbxObject* obj2 = obj0-&gt;GetSrcObject(1);</pre></div> \n                  </li>\n                  <li>Accessing the \n                     			 <em class=\"strong\">source property</em> of \n                     			 <span class=\"code\">obj0</span>: \n                     			 \n                     <div class=\"codeBlock\"><pre>// ... Assume obj0 has been initialized to reflect the diagram above...\nFbxProperty* prop0 = obj0-&gt;GetSrcProperty();</pre></div> \n                  </li>\n                  <li>Accessing the \n                     			 <em class=\"strong\">destination object</em> of \n                     			 <span class=\"code\">obj1</span>: \n                     			 \n                     <div class=\"codeBlock\"><pre>// ... assume obj1 has been initialized to reflect the diagram above...\nFbxObject* obj0 = obj1-&gt;GetDstObject();</pre></div> \n                  </li>\n                  <li>Traversing the hierarchy in an ad-hoc manner starting from \n                     			 <span class=\"code\">obj2</span>: \n                     			 \n                     <div class=\"codeBlock\"><pre>// ... Assume obj2 has been initialized to reflect the diagram above...\n// Access prop2 using obj2.\nFbxProperty* prop2 = obj2-&gt;GetSrcProperty();\n\n// For illustrative purposes, count the number of source properties of prop2.\nint numSrcProperties = prop2-&gt;GetSrcPropertyCount(); // numSrcProperties = 2\n\n// Access prop3 and prop4, which are sources with respect to prop2, \n// respectively indexed at 0 and 1.\nFbxProperty* prop3 = prop2-&gt;GetSrcProperty(0);\nFbxProperty* prop4 = prop2-&gt;GetSrcProperty(1);\n\n// Access obj0 using obj2.\nFbxObject* obj0 = obj2-&gt;GetDstObject();\n\n// Access prop0 using obj0.\nFbxProperty* prop0 = obj0-&gt;GetSrcProperty();\n\n// Access obj1 using obj0. \n// Here, we assume obj1 is indexed at 0, whereas obj2 is indexed at 1.\nFbxObject* obj1 = obj0-&gt;GetSrcObject(0);\n\n// Access prop1 using obj1.\nFbxProperty* prop1 = obj1-&gt;GetSrcProperty();</pre></div> \n                  </li>\n               </ul> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_4726F438E546419C8F21C71C4C2E07F3\"></a> \n            <h2><a name=\"GUID-C785F59A-DA2F-4EB1-BB33-B60B1C735E2D\"></a>Connecting Objects and Properties\n            </h2> \n            <p><a name=\"GUID-63C94C39-416B-4369-A456-AB62A4750682\"></a>The concept of \n               		  <em class=\"mild\">connections</em> in the FBX SDK allows for instances of \n               		  <span class=\"code\">FbxProperty</span> to be dynamically added to a \n               		  <span class=\"code\">FbxObject</span>. This gives you the flexibility of defining your own datatype, wrapping it into an\n               FBX property via the \n               		  <span class=\"code\">FbxProperty::SetUserDataPtr()</span> member function , and binding that property to an FBX object via the \n               		  <span class=\"code\">FbxObject::ConnectSrcProperty()</span> member function. \n               		\n            </p> \n            <p><a name=\"GUID-9364AEAB-56E3-4FAD-B28F-2EA0FCDF58E8\"></a> The \n               		  <span class=\"code\">FbxCollection</span> class also relies on \n               		  <em class=\"mild\">connections</em> to organize groups of objects into a hierarchy. For example, the \n               		  <span class=\"code\">FbxScene</span> class contains a hierarchy of \n               		  <span class=\"code\">FbxNode</span> objects, accessible via \n               		  <span class=\"code\">FbxScene::GetRootNode()</span>. \n               		\n            </p> \n            <p><a name=\"GUID-3627F320-F268-4D51-AF7C-0D02E22F2593\"></a>Before delving into the use of \n               		  <em class=\"mild\">connections</em> in relation to nodes (<span class=\"code\">FbxNode</span>) and node attributes (<span class=\"code\">FbxNodeAttribute</span>), it is recommended that you familiarize yourself with the scene graph organization\n               of the FBX SDK. Consult the \n               		  <span class=\"char_link\"><a href=\"GUID-F194000D-5AD4-49C1-86CC-5DAC2CE64E97.htm\">Nodes and the Scene Graph</a></span> section for more information. However, if you are feeling adventurous, the concepts\n               laid out below will give you an intuition of how the scene graph is structured using\n               nodes and their attributes. \n               		\n            </p> \n            <p><a name=\"GUID-1D4FFA39-4A82-48D9-9D12-0D2C328AE5C7\"></a>As a brief introduction, the hierarchy of \n               		  <span class=\"code\">FbxNode</span>s in a \n               		  <span class=\"code\">FbxScene</span> is used to specify the geometric transformation stack. The classes which inherit\n               from \n               		  <span class=\"code\">FbxNodeAttribute</span>, for example, \n               		  <span class=\"code\">FbxMesh</span>, \n               		  <span class=\"code\">FbxCamera</span>, and \n               		  <span class=\"code\">FbxLight</span> describe all the elements in the scene. A \n               		  <span class=\"code\">FbxNodeAttribute</span> is \n               		  <em class=\"mild\">connected</em> to a \n               		  <span class=\"code\">FbxNode</span> to specify where the mesh, camera, or light exists in the 3D space. \n               		\n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_26DB8FF33D5D46ECA980697D869242D2\"></a> \n            <h2><a name=\"GUID-04F55A8B-B40F-403F-BBD6-732B34242DCB\"></a>\"Object-Object\" Connection Example: Parent-Child Relationships Between Nodes in a\n               Scene\n            </h2> \n            <p><a name=\"GUID-AFAD02D2-6A96-4C63-87B1-8B70DEBC134A\"></a>The parent-child relationships between nodes in a scene make use of \n               		  <em class=\"strong\">object</em> connections. Consider the \n               		  <span class=\"code\">FbxNode::AddChild()</span> method, which adds a child node to the parent node on which the method is called:\n               \n               		  \n               <div class=\"codeBlock\"><pre>// ... Assume lScene has been initialized as a FbxScene*,\n\n// Obtain the root node of a scene.\nFbxNode* lParentNode = lScene-&gt;GetRootNode();\n\n// Create a child node.\nFbxNode* lChildNode = FbxNode::Create(lScene, \"child\");\n\n// Add the child node to the root node.\nlParentNode-&gt;AddChild(lChildNode);</pre></div> \n            </p> \n            <p><a name=\"GUID-99F99E63-9C5D-4156-B2AF-59E6E1845E7A\"></a> <a name=\"UL_5D1BC0EEA6014F60BB016C3885F58429\"></a><p class=\"list-intro\"><a name=\"GUID-223B2DFF-A769-4BD0-8265-A1EB73AEAC27\"></a>The following \n                  		  <em class=\"mild\">connections</em> will be made:\n               </p>\n               <ul>\n                  <li>The child node will be a \n                     			 <em class=\"strong\">source object</em> of the parent node. \n                     		  \n                  </li>\n                  <li>The parent node will be the \n                     			 <em class=\"strong\">destination object</em> of the child node. \n                     		  \n                  </li>\n               </ul> \n            </p> \n            <div><a name=\"GUID-15AF5C21-7E25-4E13-A82A-E31D6A34EFBD\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span><span class=\"code\">lParentNode</span> is a \n                  		  <em class=\"strong\">source object</em> of \n                  		  <span class=\"code\">lScene</span>. Thus, \n                  		  <span class=\"code\">lScene</span> is the \n                  		  <em class=\"strong\">destination object</em> of \n                  		  <span class=\"code\">lParentNode</span>. \n                  		\n               </div>\n            </div> \n            <div><a name=\"GUID-A4C4AD64-C7EA-4C5A-93D8-053C2824BFBC\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The \n                  		  <span class=\"char_link\"><a href=\"GUID-C08BAE05-B074-441C-9DBE-CE1811C0E6E2.htm\">Merging Two Scenes</a></span> topic explicitly manipulates the connections between nodes to merge the contents\n                  of two scenes. \n                  		\n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_94CD0661C7E94C90ACA8E089E9BEABE4\"></a> \n            <h2><a name=\"GUID-775E12CE-D6EC-4EB1-A64A-4886C3E35BA8\"></a>\"Object-Object\" Connection Example: Nodes and Node Attributes\n            </h2> \n            <p><a name=\"GUID-FEA73461-7E8B-4B19-AD49-9C42A2DBAC27\"></a>A \n               		  <span class=\"code\">FbxNode</span>\'s relationship to a \n               		  <span class=\"code\">FbxNodeAttribute</span> is normally created by invoking \n               		  <span class=\"code\">FbxNode::SetNodeAttribute()</span>. As such, an instance of \n               		  <span class=\"code\">FbxMesh</span> (which inherits from \n               		  <span class=\"code\">FbxNodeAttribute</span>) can be bound to a node in a scene. In this case: \n               		<a name=\"UL_36AFCE2754B94CFF908F61B909A5B3F8\"></a><ul>\n                  <li>The \n                     			 <span class=\"code\">FbxNodeAttribute</span> is a \n                     			 <em class=\"strong\">source object</em> of the \n                     			 <span class=\"code\">FbxNode</span>. \n                     		  \n                  </li>\n                  <li>The \n                     			 <span class=\"code\">FbxNode</span> is a \n                     			 <em class=\"strong\">destination object</em> of the \n                     			 <span class=\"code\">FbxNodeAttribute</span>. \n                     		  \n                  </li>\n               </ul> \n            </p> \n            <div><a name=\"GUID-90B00754-D9EF-4937-A448-14EDC8EA1160\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>Materials (<span class=\"code\">FbxSurfaceMaterial</span>) are also connected as \n                  		  <em class=\"strong\">source objects</em> to \n                  		  <span class=\"code\">FbxNode</span>s. One node can be connected to many materials, and one material can be connected\n                  to many nodes (to reduce memory usage). Observe, however, that \n                  		  <span class=\"code\">FbxSurfaceMaterial</span> is not a subclass of \n                  		  <span class=\"code\">FbxNodeAttribute</span>. \n                  		\n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_9E5E27BADFAF45D0843CE54E7DDE09A7\"></a> \n            <h2><a name=\"GUID-5E5DD61A-D56E-4E46-BCF2-7AC72734FBF4\"></a>\"Object-Property\" Connection Example: Nodes and Transformations\n            </h2> \n            <p><a name=\"GUID-74D948C6-9380-4A5E-B186-2B4BE362377F\"></a>As mentioned in the \n               		  <span class=\"char_link\"><a href=\"GUID-283ED424-E506-4A29-848F-9EAC8F1C4AF9.htm#SECTION_1E7A9F74F33F4D0CA2C167A520839792\">Property Data</a></span> subsection, the local transformation data of a \n               		  <span class=\"code\">FbxNode</span> is defined as a \n               		  <span class=\"code\">FbxPropertyT</span>, parametrized with the \n               		  <span class=\"code\">FbxDouble3</span> datatype. In this case: \n               		<a name=\"UL_2C9865313E0D4B9A8470B98F54540E38\"></a><ul>\n                  <li>The \n                     			 <span class=\"code\">FbxPropertyT</span> returned by \n                     			 <span class=\"code\">FbxNode::LclTranslation</span> is a \n                     			 <em class=\"strong\">source property</em> of that \n                     			 <span class=\"code\">FbxNode</span>. \n                     		  \n                  </li>\n                  <li>The \n                     			 <span class=\"code\">FbxNode</span> is a \n                     			 <em class=\"strong\">destination object</em> of that \n                     			 <span class=\"code\">FbxPropertyT</span>. \n                     		  \n                  </li>\n               </ul> \n            </p> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";