var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-E5C89F5F-09D1-4C32-BA83-28F63D0B2A6C\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Animating a Node</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-E5C89F5F-09D1-4C32-BA83-28F63D0B2A6C\"></a><div class=\"head\">\n            <h1>Animating a Node</h1>\n         </div>\n         <p><a name=\"WS1A9193826455F5FF453265C9125FAA23BBB-1689\"></a>Setting up the data structures for animation is simple if you are not using blended\n            animation. You need at least one animation stack, one animation layer, and one animation\n            curve node for each FBX property. \n         </p>\n         <p><a name=\"WS1A9193826455F5FF45564F421269B08A8E9-56AB\"></a>This example shows how to animate the local translation property of a camera’s <span class=\"code\">FbxNode</span> object. \n         </p>\n         <div class=\"codeBlock\"><pre>\nFbxScene* myScene;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// An initialized and valid scene object\nFbxNode* myCameraNode;&nbsp;&nbsp;&nbsp;&nbsp;// An initialized and valid node object\n&nbsp;\n// Set the default local translation values (X, Y, Z) for the camera\nmyCameraNode.LclTranslation.Set(fbxDouble3(1.1, 2.2, 3.3))\n</pre></div>\n         <p><a name=\"WS73099CC142F48755-191A86AC126720FA3543F7A\"></a>Although, this example shows how to animate the translation property of a camera,\n            you can animate any FBX property of any FBX object, if the FBX property is <em class=\"mild\">animatable</em>. See <span class=\"code\">FbxPropertyFlags::eAnimatable</span> for more information. \n         </p>\n         <p><a name=\"GUID-6A87B0F9-DCBA-4F1F-8AC3-E6A1E89548DA\"></a>Follow the instructions in these sections to animate the local translation property\n            of the camera. \n         </p>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-191A86AC126B843A87B-61BF\"></a> \n            <h2><a name=\"GUID-3516E2E3-3616-4DA5-9B78-F689BF5D6466\"></a>Start by creating the Animation stack and Animation Layer\n            </h2> <a name=\"OL_6AE07AEE52A147EEB18FC909A71FA9A7\"></a><p class=\"list-intro\"><a name=\"GUID-5454181A-D499-420B-9FD1-5822F98886DA\"></a>Perform the following steps:\n            </p>\n            <ol type=\"1\" start=\"1\">\n               <li>Create one animation stack. \n                  <div class=\"codeBlock\"><pre>\n// Create an animation stack\nFbxAnimStack* myAnimStack = FbxAnimStack::Create(pScene, \"My stack\");\n</pre></div> \n               </li>\n               <li>Create one animation layer. \n                  <div class=\"codeBlock\"><pre>\n// Create the base layer (this is mandatory)\nFbxAnimLayer* myAnimBaseLayer = FbxAnimLayer::Create(pScene, \"Layer0\");\n</pre></div> \n               </li>\n               <li>Add the base layer to the animation stack. \n                  <div class=\"codeBlock\"><pre>\nmyAnimStack-&gt;AddMember(myAnimBaseLayer);\n</pre></div> \n               </li>\n            </ol> \n            <p><a name=\"GUID-85D9ABC4-6487-4AE7-BDE6-5B1CD162D29A\"></a>After adding the base layer to the animation stack, you can create the animation curve\n               nodes to connect the animation data to the FBX property. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-191A86AC126B843A87B-61BE\"></a> \n            <h2><a name=\"GUID-959C1229-3B78-4626-83ED-00E4A6276042\"></a>Create the Animation Curve Nodes\n            </h2> \n            <p><a name=\"WS1A9193826455F5FF45564F421269B08A8E9-56AA\"></a>The CubeCreator tutorial program creates one animation curve node for each axis along\n               which the camera moves. \n            </p> \n            <p><a name=\"GUID-E940E5E0-8C1E-4CAB-915D-5FD62ADAC5BD\"></a>Create one animation curve node to connect the translation data to the camera node’s\n               <span class=\"code\">LclTranslation</span> property: \n            </p> \n            <div class=\"codeBlock\"><pre>\n// Get the camera’s curve node for local translation.\n// The second parameter to GetCurveNode() is \"true\" to ensure\n// that the curve node is automatically created, if it does not exist.\nFbxAnimCurveNode* myAnimCurveNode = pCamera-&gt;LclTranslation.GetCurveNode(pAnimLayer, true);\n</pre></div> <a name=\"WS1A9193826455F5FF45564F421269B08A8E9-56A9\"></a><p class=\"list-intro\"><a name=\"WS1A9193826455F5FF45564F421269B08A8E9-56A8\"></a>Notes:\n            </p>\n            <ul>\n               <li> <span class=\"code\">myAnimCurveNode</span> is automatically created with three animation channels of type <span class=\"code\">double</span> because the type of <span class=\"code\">lclTranslation</span> is <span class=\"code\">fbxDouble3</span>. See <span class=\"code\">FbxNode::LclTranslation</span>. \n               </li>\n               <li> <span class=\"code\">myAnimCurveNode</span> is automatically connected to <span class=\"code\">myCameraNode.LclTranslation</span>, which is the local translation property of the camera. \n               </li>\n               <li> <span class=\"code\">myAnimCurveNode</span> is automatically added as a member curve node to <span class=\"code\">myAnimBaseLayer</span>. Animation layers are containers for animation curve nodes. \n               </li>\n            </ul>\n            <p><a name=\"GUID-BE22FDB5-681C-4F3B-B1EA-4A920F64F4AF\"></a>For another approach to create animation curve nodes and animation curves, see the\n               animation sample program in <span class=\"code\"><em class=\"mild\">&lt;yourFBXSDKpath&gt;</em>\\samples\\Animation\\</span>. \n            </p> \n            <p><a name=\"WS73099CC142F48755-191A86AC126B843A87B-61E4\"></a> After the animation curve node (<span class=\"code\">myAnimCurveNode</span>) is set up to connect the animation curves to the local translation property of the\n               camera (<span class=\"code\">myCameraNode</span>), you can create the animation curves. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-191A86AC126B843A87B-61BD\"></a> \n            <h2><a name=\"GUID-4468CE0E-3C3B-4D02-92BD-7D63649B7BD5\"></a>Create the Animation curves\n            </h2> \n         </div><a name=\"OL_FAB5E6FA94374B10A1607CA1FCE6FE5D\"></a><p class=\"list-intro\"><a name=\"GUID-954E00B9-2F6F-4E60-91A1-53005CD5DEE2\"></a>Perform the following steps:\n         </p>\n         <ol type=\"1\" start=\"1\">\n            <li>Create the animation curve for translation of the camera on the X-axis. \n               <div class=\"codeBlock\"><pre>\nFbxAnimCurve* myTranXCurve = NULL;&nbsp;&nbsp;&nbsp;// Curve for local translation on X-axis\nFbxTime myTime;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// For the start and stop keys.\nint myKeyIndex = 0;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Index for the keys that define the curve\n&nbsp;\n// Get the animation curve for local translation of the camera.\n// true: If the curve does not exist yet, create it.\nmyTranXCurve = myCameraNode-&gt;LclTranslation.GetCurve(myAnimLayer, KFCURVENODE_T_X, true);\n</pre></div> \n            </li>\n            <li>Define the starting and ending keyframes of the animation curve. \n               <div class=\"codeBlock\"><pre>\n// First the start keyframe\nmyAnimCurve-&gt;KeyModifyBegin();\nmyTime.SetSecondDouble(0.0);&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Starting time\nmyKeyIndex = myAnimCurve-&gt;KeyAdd(lTime); // Add the start key; returns 0\n&nbsp;\nmyAnimCurve-&gt;KeySet(lKeyIndex,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Set the zero’th key\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;lTime,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Starting time\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;0.0,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Starting X value\nFbxAnimCurveDef::eInterpolationLinear);// Straight line between 2 points\n&nbsp;\n// Then the stop keyframe\nlTime.SetSecondDouble(20.0);\nlKeyIndex = myAnimCurve-&gt;KeyAdd(lTime);\nmyAnimCurve-&gt;KeySet(lKeyIndex, lTime, 500.0, FbxAnimCurveDef::eInterpolationLinear);\nmyAnimCurve-&gt;KeyModifyEnd();\n</pre></div> \n            </li>\n         </ol>\n         <p><a name=\"WS73099CC142F48755-191A86AC126B843A87B-61BB\"></a>If you run this animation, the camera takes 20 seconds to move 500 units along the\n            X-axis. \n         </p><a name=\"WS1A9193826455F5FF45564F421269B08A8E9-4D88\"></a><p class=\"list-intro\"><a name=\"WS73099CC142F48755-191A86AC126B843A87B-61E2\"></a>You can refer to these sample programs that demonstrate animation:\n         </p>\n         <ul>\n            <li>The Animation sample program in <span class=\"code\"><em class=\"mild\">&lt;yourFBXSDKpath&gt;</em>\\samples\\Animation\\</span>. \n            </li>\n            <li>The CubeCreator tutorial program in <span class=\"code\"><em class=\"mild\">&lt;yourFBXSDKpath&gt;</em>\\samples\\UI&nbsp;Examples\\CubeCreator\\</span>. \n            </li>\n         </ul>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li><span class=\"char_link\"><a href=\"GUID-E8A5224C-488F-4477-AFE1-FC1EDB6934CB.htm\">Animation Data Structures </a></span></li>\n                  <li><span class=\"char_link\"><a href=\"GUID-5A05DFB2-4985-4FE5-B0EF-C2855F925AAF.htm\">Cameras </a></span></li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";