var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2017\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-03-15\">\n      <meta name=\"topicid\" content=\"GUID-FCD71AFF-F753-4D58-B96E-453ED84BF5C4\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>FBX Node Attributes</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-FCD71AFF-F753-4D58-B96E-453ED84BF5C4\"></a><div class=\"head\">\n            <h1>FBX Node Attributes</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_C8939044962945BCA849D71BBBDEA077\"></a> \n            <h2><a name=\"GUID-65B02E53-2666-4106-BF14-70FF6E8BB8AE\"></a>Creating a Node Attribute\n            </h2> \n            <p><a name=\"GUID-09630EF3-039F-4E93-9D0D-9A0DBFD57A8D\"></a>A <span class=\"code\">FbxNodeAttribute</span> is paired with a <span class=\"code\">FbxNode</span> to define a scene element with a specific position, rotation and scale. Calling <span class=\"code\">FbxNode::GetNodeAttribute()</span> will return <span class=\"code\">NULL</span> if no node attribute was set for that node. \n            </p> \n            <p><a name=\"GUID-984741DA-17BE-45E9-A0D5-B92A4B2669BE\"></a></p> <img src=\"../images/GUID-F77410FB-73C9-40F5-A571-19EB93ECE418-low.png\"> \n            <p><a name=\"GUID-D132AA03-D56A-480C-AE7C-D894F4A29CFB\"></a>The following code sample (adapted from <span class=\"code\">ExportScene04/main.cxx</span> to fit the diagram above) illustrates how to create a simple spotlight within a scene.\n               Here, <span class=\"code\">FbxLight* light</span> is the node attribute of <span class=\"code\">FbxNode* lightNode</span>. For more information on lights, see <span class=\"char_link\"><a href=\"GUID-2F4395F9-5BB3-413C-BCC4-C519DAF6A484.htm\">Lights</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a spotlight. \nFbxNode* CreateLight(FbxScene* pScene, char* pName)\n{\n    FbxLight* light = FbxLight::Create(pScene,pName);\n\n    light-&gt;LightType.Set(FbxLight::eSpot);\n    light-&gt;CastLight.Set(true);\n\n    FbxNode* lightNode = FbxNode::Create(pScene,pName);\n\n    lightNode-&gt;SetNodeAttribute(light);\n\n    return lightNode;\n}</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_D3DBF1041E314108A9847BCF2020679B\"></a> \n            <h2><a name=\"GUID-2F5BCFF4-55C2-4637-AAA2-E7417FF3F508\"></a>Node Attributes\n            </h2> \n            <p><a name=\"GUID-59E11571-C5FB-401A-85D4-B50390E72E89\"></a>The following table presents a set of basic scene elements and their related <span class=\"code\">FbxNodeAttribute</span> subclasses. These node attributes can be paired with a <span class=\"code\">FbxNode</span> using <span class=\"code\">FbxNode::SetNodeAttribute()</span>. Consult the C++ Reference guide for the full class hierarchy. \n            </p> <a name=\"WS1A9193826455F5FF-150B16DA11960D83164-6C23\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"28.729066597429576%\">\n                     <col width=\"71.27093340257042%\">\n                  </colgroup>\n                  <tr class=\"ruled-heading\">\n                     <th class=\"table-heading\">Scene Element </th>\n                     <th class=\"table-heading\"><span class=\"code\">FbxNodeAttribute</span> subclass \n                     </th>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\">Camera </td>\n                     <td class=\"table-body\"><span class=\"code\">FbxCamera</span>, <span class=\"code\">FbxCameraStereo</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">Camera Switcher (custom camera definition in Autodesk MotionBuilder) </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxCameraSwitcher</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\">Light </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxLight</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">Mesh </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxMesh</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\">Nurb </td>\n                     <td class=\"table-body\"><span class=\"code\">FbxNurbs</span>, <span class=\"code\">FbxNurbsCurve</span>, <span class=\"code\">FbxNurbsSurface</span>, <span class=\"code\">FbxTrimNurbsSurface</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">Patch / Parametric Surface </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxPatch</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\">Level of Detail Group </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxLodGroup</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">Marker </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxMarker</span> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\">Skeleton </td>\n                     <td class=\"table-body\"> <span class=\"code\">FbxSkeleton</span> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"GUID-EF56CBF1-E182-4D67-ADFA-6B9C1AB818C4\"></a> \n               <div><a name=\"GUID-06F038A2-C189-4BA6-9329-04072EBE3409\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>Some applications require a null node type in their scene graph. The <span class=\"code\">FbxNull</span> node attribute is used to define such a node type. Observe that an instance of <span class=\"code\">FbxNull</span> is not the same as the <span class=\"code\">NULL</span> value. \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_7284FBD36B3340ADA220FAB7A517DD04\"></a> \n            <h2><a name=\"GUID-BDB01F52-9686-42CC-80ED-AE0D3A102DA0\"></a>Node Attribute Type\n            </h2> \n            <p><a name=\"GUID-60B68D09-0FA4-465C-83DA-68010158CCB7\"></a>The type (<span class=\"code\">FbxNodeAttribute::EAttributeType</span>) of a <span class=\"code\">FbxNodeAttribute</span> can be obtained by calling <span class=\"code\">FbxNodeAttribute::GetAttributeType()</span>. The <span class=\"code\">EAttributeType</span> is useful for downcasting the node attribute object to its appropriate subclass.\n               \n            </p> \n            <p><a name=\"GUID-84A2FF8B-0568-495C-A410-85D2E0B2C0C2\"></a>The following code sample (adapted from <span class=\"code\">ImportScene/main.cxx</span> and <span class=\"code\">ImportScene/DisplayLight.cxx</span>) illustrates how to use a switch to display a <span class=\"code\">FbxLight</span> contained in a <span class=\"code\">FbxNode</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>//\n// Adapted from ImportScene/DisplayLight.cxx ...\n// Display the various properties of the FbxLight contained within the FbxNode.\n//\nvoid DisplayLight(FbxNode* pNode)\n{\n    FbxLight* lLight = (FbxLight*) pNode-&gt;GetNodeAttribute();\n\n    DisplayString(\"Light Name: \", (char *) pNode-&gt;GetName());\n    // ...\n\n    char* lLightTypes[] = { \"Point\", \"Directional\", \"Spot\" };\n\n    DisplayString(\"    Type: \", lLightTypes[lLight-&gt;LightType.Get()]);\n    DisplayBool(\"    Cast Light: \", lLight-&gt;CastLight.Get());\n\n    if (!(lLight-&gt;FileName.Get().IsEmpty()))\n    {\n        DisplayString(\"    Gobo\");\n\n        DisplayString(\"        File Name: \\\"\", lLight-&gt;FileName.Get().Buffer(), \"\\\"\");\n        DisplayBool(\"        Ground Projection: \", lLight-&gt;DrawGroundProjection.Get());\n        DisplayBool(\"        Volumetric Projection: \", lLight-&gt;DrawVolumetricLight.Get());\n        DisplayBool(\"        Front Volumetric Projection: \", lLight-&gt;DrawFrontFacingVolumetricLight.Get());\n    }\n\n    // ...\n}\n\n//\n// Adapted from ImportScene/main.cxx ...\n// Display the contents of a node. Here, we are only interested in\n// looking  at the eLight attribute type, which coincides with the \n// FbxLight node attribute.\n//\nvoid DisplayContent(FbxNode* pNode)\n{\n    FbxNodeAttribute::EAttributeType lAttributeType;\n    int i;\n\n    if(pNode-&gt;GetNodeAttribute() == NULL)\n    {\n        printf(\"NULL Node Attribute\\n\\n\");\n    }\n    else\n    {\n        lAttributeType = (pNode-&gt;GetNodeAttribute()-&gt;GetAttributeType());\n\n        switch (lAttributeType)\n        {\n            // ...\n\n        case FbxNodeAttribute::eLight:     \n            DisplayLight(pNode);\n            break;\n\n            // ...\n        }\n    }\n\n    // ...\n\n    for(i = 0; i &lt; pNode-&gt;GetChildCount(); i++)\n    {\n        DisplayContent(pNode-&gt;GetChild(i));\n    }\n}</pre></div> \n         </div><span class=\"noindex\">\n            <div class=\"topic-list\">\n               <h4 class=\"related\">Topics in this section</h4>\n               <ul class=\"jumplist\">\n                  <li class=\"topiclist-litem\"><a href=\"GUID-2F4395F9-5BB3-413C-BCC4-C519DAF6A484.htm\">Lights</a></li>\n                  <li class=\"topiclist-litem\"><a href=\"GUID-5A05DFB2-4985-4FE5-B0EF-C2855F925AAF.htm\">Cameras</a></li>\n                  <li class=\"topiclist-litem\"><a href=\"GUID-4665F563-661B-4FC7-B867-2D1AE8032A1F.htm\">Geometry</a></li>\n               </ul>\n            </div></span><div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";